@@ -19,23 +19,6 @@ pub type MicroKernelFn<T> = unsafe fn(
     *const T,
 );
 
-pub type HMicroKernelFn<T> = unsafe fn(
-    k: usize,
-    dst: *mut T,
-    lhs: *const T,
-    rhs: *const T,
-    dst_cs: isize,
-    dst_rs: isize,
-    lhs_rs: isize,
-    rhs_cs: isize,
-    alpha: T,
-    beta: T,
-    alpha_status: u8,
-    _conj_dst: bool,
-    _conj_lhs: bool,
-    _conj_rhs: bool,
-);
-
 // microkernel_fn_array!{
 // [ a, b, c, ],
 // [ d, e, f, ],
@@ -110,21 +93,6 @@ macro_rules! microkernel_fn_array {
     };
 }
 
-#[macro_export]
-macro_rules! hmicrokernel_fn_array {
-    ($([
-       $($ukr: ident,)*
-    ],)*) => {
-       pub const H_M: usize =
-           $crate::__microkernel_fn_array_helper!([$([$($ukr,)*],)*]);
-       pub const H_N: usize =
-           $crate::__microkernel_fn_array_helper_nr!($([$($ukr,)*],)*);
-
-        pub const H_UKR: [[$crate::microkernel::HMicroKernelFn<T>; H_N]; H_M] =
-            [ $([$($ukr,)*]),* ];
-    };
-}
-
 #[macro_export]
 macro_rules! microkernel_cplx_fn_array {
     ($([
@@ -140,21 +108,6 @@ macro_rules! microkernel_cplx_fn_array {
     };
 }
 
-#[macro_export]
-macro_rules! hmicrokernel_cplx_fn_array {
-    ($([
-       $($ukr: ident,)*
-    ],)*) => {
-       pub const H_CPLX_M: usize =
-           $crate::__microkernel_fn_array_helper!([$([$($ukr,)*],)*]);
-       pub const H_CPLX_N: usize =
-           $crate::__microkernel_fn_array_helper_nr!($([$($ukr,)*],)*);
-
-        pub const H_CPLX_UKR: [[$crate::microkernel::HMicroKernelFn<num_complex::Complex<T>>; H_CPLX_N]; H_CPLX_M] =
-            [ $([$($ukr,)*]),* ];
-    };
-}
-
 #[macro_export]
 macro_rules! amx {
     ($op: tt, $gpr: expr) => {
@@ -166,6 +119,7 @@ macro_rules! amx {
 #[macro_export]
 macro_rules! microkernel_amx {
     ($ty: tt, $([$target: tt])?, $unroll: tt, $name: ident, $mr_div_n: tt, $nr: tt , $nr_div_n: tt, $n: tt) => {
+        #[inline]
         $(#[target_feature(enable = $target)])?
         // 0, 1, or 2 for generic alpha
         pub unsafe fn $name(
@@ -397,6 +351,7 @@ macro_rules! microkernel_amx {
 #[macro_export]
 macro_rules! microkernel {
     ($([$target: tt])?, $unroll: tt, $name: ident, $mr_div_n: tt, $nr: tt $(, $nr_div_n: tt, $n: tt)?) => {
+        #[inline]
         $(#[target_feature(enable = $target)])?
         // 0, 1, or 2 for generic alpha
         pub unsafe fn $name(
@@ -711,6 +666,7 @@ macro_rules! microkernel {
 #[macro_export]
 macro_rules! microkernel_cplx_2step {
     ($([$target: tt])?, $unroll: tt, $name: ident, $mr_div_n: tt, $nr: tt) => {
+        #[inline]
         $(#[target_feature(enable = $target)])?
         // 0, 1, or 2 for generic alpha
         pub unsafe fn $name(
@@ -1136,6 +1092,7 @@ macro_rules! microkernel_cplx_2step {
 #[macro_export]
 macro_rules! microkernel_cplx {
     ($([$target: tt])?, $unroll: tt, $name: ident, $mr_div_n: tt, $nr: tt) => {
+        #[inline]
         $(#[target_feature(enable = $target)])?
         // 0, 1, or 2 for generic alpha
         pub unsafe fn $name(
@@ -1518,6 +1475,7 @@ macro_rules! microkernel_cplx {
 #[macro_export]
 macro_rules! microkernel_cplx_packed {
     ($([$target: tt])?, $unroll: tt, $name: ident, $mr_div_n: tt, $nr: tt) => {
+        #[inline]
         $(#[target_feature(enable = $target)])?
         // 0, 1, or 2 for generic alpha
         pub unsafe fn $name(
