@@ -1,9 +1,24 @@
-//! A fast & efficient ordered set for unsigned integers.
+//! A fast, efficient, sparse, & ordered unsigned integer (`u32`) bit set.
+//!
+//! There are a couple of differences with [`super::IntSet`]:
+//! - This set is not invertible and can only record the set of integers which are members.
+//! - This set works only with `u32` values, unlike [`super::IntSet`] which supports custom integer types.
+//!
+//! When dealing with only `u32`'s and invertibility is not needed then this set is slightly faster
+//! than the more generic [`super::IntSet`].
+//!
+//! The bitset is implemented using fixed size pages which allows it to compactly
+//! represent sparse membership. However, the set excels when set members are typically
+//! clustered together. For example when representing glyph id or unicode codepoint values
+//! in a font.
+//!
+//! When constructing a new [`U32Set`] from an existing list of integer values the most efficient
+//! way to create the set is to initialize it from a sorted list of values via the extend() method.
 
 use super::bitpage::BitPage;
 use super::bitpage::RangeIter;
 use super::bitpage::PAGE_BITS;
-use std::cell::Cell;
+use core::sync::atomic::AtomicUsize;
 use std::cmp::Ordering;
 use std::hash::Hash;
 
@@ -12,26 +27,71 @@ use std::ops::RangeInclusive;
 // log_2(PAGE_BITS)
 const PAGE_BITS_LOG_2: u32 = PAGE_BITS.ilog2();
 
-/// An ordered integer (u32) set.
-#[derive(Clone, Debug)]
-pub(crate) struct BitSet {
+/// A fast, efficient, sparse, & ordered `u32` set.
+///
+/// For a higher-level API that supports inversion and generic int types, use [`super::IntSet`]
+#[derive(Debug)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+pub struct U32Set {
     // TODO(garretrieger): consider a "small array" type instead of Vec.
     pages: Vec<BitPage>,
     page_map: Vec<PageInfo>,
-    len: Cell<u64>, // TODO(garretrieger): use an option instead of a sentinel.
+    length: u64,
+
+    #[cfg_attr(feature = "serde", serde(skip))]
+    #[cfg_attr(feature = "serde", serde(default = "default_last_page_map_index"))]
+    last_page_map_index: AtomicUsize,
 }
 
-impl BitSet {
+const fn default_last_page_map_index() -> AtomicUsize {
+    AtomicUsize::new(usize::MAX)
+}
+
+impl Default for U32Set {
+    fn default() -> Self {
+        Self {
+            pages: Default::default(),
+            page_map: Default::default(),
+            length: Default::default(),
+            last_page_map_index: default_last_page_map_index(),
+        }
+    }
+}
+
+impl Clone for U32Set {
+    fn clone(&self) -> Self {
+        Self {
+            pages: self.pages.clone(),
+            page_map: self.page_map.clone(),
+            length: self.length,
+            // last_page_map_index has no effect on the externally visible state of the set
+            // so it can just be reset to the default value.
+            last_page_map_index: default_last_page_map_index(),
+        }
+    }
+}
+
+impl FromIterator<u32> for U32Set {
+    fn from_iter<I: IntoIterator<Item = u32>>(iter: I) -> Self {
+        let mut s = U32Set::empty();
+        s.extend(iter);
+        s
+    }
+}
+
+impl U32Set {
     /// Add val as a member of this set.
-    pub(crate) fn insert(&mut self, val: u32) -> bool {
+    ///
+    /// If the set did not previously contain this value, returns `true`.
+    pub fn insert(&mut self, val: u32) -> bool {
         let page = self.ensure_page_for_mut(val);
         let ret = page.insert(val);
-        self.mark_dirty();
+        self.length += ret as u64;
         ret
     }
 
     /// Add all values in range as members of this set.
-    pub(crate) fn insert_range(&mut self, range: RangeInclusive<u32>) {
+    pub fn insert_range(&mut self, range: RangeInclusive<u32>) {
         let start = *range.start();
         let end = *range.end();
         if start > end {
@@ -41,34 +101,43 @@ impl BitSet {
         let major_start = Self::get_major_value(start);
         let major_end = Self::get_major_value(end);
 
+        let mut total_added = 0;
+
         for major in major_start..=major_end {
             let page_start = start.max(Self::major_start(major));
             let page_end = end.min(Self::major_start(major) + (PAGE_BITS - 1));
             let page = self.ensure_page_for_major_mut(major);
+            let pre_len = page.len();
             page.insert_range(page_start, page_end);
+            let delta_len = page.len() - pre_len;
+            total_added += delta_len as u64;
         }
-        self.mark_dirty();
+        self.length += total_added;
     }
 
     /// An alternate version of [`extend()`] which is optimized for inserting an unsorted
     /// iterator of values.
     ///
     /// [`extend()`]: Self::extend
-    pub(crate) fn extend_unsorted<U: IntoIterator<Item = u32>>(&mut self, iter: U) {
-        for val in iter {
-            let major_value = Self::get_major_value(val);
-            let page = self.ensure_page_for_major_mut(major_value);
-            page.insert_no_return(val);
-        }
-        self.mark_dirty();
+    pub fn extend_unsorted<U: IntoIterator<Item = u32>>(&mut self, iter: U) {
+        self.length += iter
+            .into_iter()
+            .map(|val| {
+                let major_value = Self::get_major_value(val);
+                let page = self.ensure_page_for_major_mut(major_value);
+                page.insert(val) as u64
+            })
+            .sum::<u64>();
     }
 
     /// Remove val from this set.
-    pub(crate) fn remove(&mut self, val: u32) -> bool {
+    ///
+    /// Returns `true` if the value was present.
+    pub fn remove(&mut self, val: u32) -> bool {
         let maybe_page = self.page_for_mut(val);
         if let Some(page) = maybe_page {
             let ret = page.remove(val);
-            self.mark_dirty();
+            self.length -= ret as u64;
             ret
         } else {
             false
@@ -76,9 +145,10 @@ impl BitSet {
     }
 
     // Remove all values in iter from this set.
-    pub(crate) fn remove_all<U: IntoIterator<Item = u32>>(&mut self, iter: U) {
+    pub fn remove_all<U: IntoIterator<Item = u32>>(&mut self, iter: U) {
         let mut last_page_index: Option<usize> = None;
         let mut last_major_value = u32::MAX;
+        let mut total_removed = 0;
         for val in iter {
             let major_value = Self::get_major_value(val);
             if major_value != last_major_value {
@@ -91,14 +161,14 @@ impl BitSet {
             };
 
             if let Some(page) = self.pages.get_mut(page_index) {
-                page.remove(val);
+                total_removed += page.remove(val) as u64;
             }
         }
-        self.mark_dirty();
+        self.length -= total_removed;
     }
 
     /// Removes all values in range as members of this set.
-    pub(crate) fn remove_range(&mut self, range: RangeInclusive<u32>) {
+    pub fn remove_range(&mut self, range: RangeInclusive<u32>) {
         let start = *(range.start());
         let end = *(range.end());
         if start > end {
@@ -136,47 +206,97 @@ impl BitSet {
             info_index += 1;
         }
 
-        self.mark_dirty();
+        self.recompute_length();
     }
 
     /// Returns true if val is a member of this set.
-    pub(crate) fn contains(&self, val: u32) -> bool {
-        self.page_for(val)
+    pub fn contains(&self, val: u32) -> bool {
+        let new_major = U32Set::get_major_value(val);
+
+        let lookup_result = self
+            .page_map
+            .get(
+                self.last_page_map_index
+                    .load(std::sync::atomic::Ordering::Relaxed),
+            )
+            .filter(|info| info.major_value == new_major)
+            .map(|info| Some(info.index as usize))
+            .unwrap_or(None);
+
+        let page_index = match lookup_result {
+            None => {
+                // Cached value needs an update, lookup the actual page map index.
+                let Some(page_map_index) = self.page_map_index_for_major(new_major) else {
+                    // No page exists for this value so it's not present and we don't need to update cached values.
+                    return false;
+                };
+
+                self.last_page_map_index
+                    .store(page_map_index, std::sync::atomic::Ordering::Relaxed);
+                self.page_map[page_map_index].index as usize
+            }
+            Some(page_index) => page_index,
+        };
+
+        self.pages
+            .get(page_index)
             .map(|page| page.contains(val))
             .unwrap_or(false)
     }
 
-    pub(crate) fn empty() -> BitSet {
-        BitSet {
-            pages: vec![],
-            page_map: vec![],
-            len: Default::default(),
+    pub fn intersects_set(&self, other: &U32Set) -> bool {
+        let mut it_a = self.page_map.iter().peekable();
+        let mut it_b = other.page_map.iter().peekable();
+
+        while let (Some(a), Some(b)) = (it_a.peek(), it_b.peek()) {
+            match a.major_value.cmp(&b.major_value) {
+                Ordering::Equal => {
+                    if self.pages[a.index as usize].intersects_set(&other.pages[b.index as usize]) {
+                        return true;
+                    }
+                    it_a.next();
+                    it_b.next();
+                }
+                Ordering::Less => {
+                    it_a.next();
+                }
+                Ordering::Greater => {
+                    it_b.next();
+                }
+            }
+        }
+
+        false
+    }
+
+    pub const fn empty() -> U32Set {
+        U32Set {
+            pages: Vec::new(),
+            page_map: Vec::new(),
+            length: 0,
+            last_page_map_index: default_last_page_map_index(),
         }
     }
 
     /// Remove all members from this set.
-    pub(crate) fn clear(&mut self) {
+    pub fn clear(&mut self) {
         self.pages.clear();
         self.page_map.clear();
-        self.mark_dirty();
+        self.length = 0;
     }
 
     /// Return true if there are no members in this set.
-    #[cfg(test)]
-    pub(crate) fn is_empty(&self) -> bool {
+    pub fn is_empty(&self) -> bool {
         self.len() == 0
     }
 
+    fn recompute_length(&mut self) {
+        self.length = self.pages.iter().map(|page| page.len() as u64).sum();
+    }
+
     /// Returns the number of members in this set.
-    pub(crate) fn len(&self) -> u64 {
-        // TODO(garretrieger): keep track of len on the fly, rather than computing it. Leave a computation method
-        //                     for complex cases if needed.
-        if self.is_dirty() {
-            // this means we're stale and should recompute
-            let len = self.pages.iter().map(|val| val.len() as u64).sum();
-            self.len.set(len);
-        }
-        self.len.get()
+    pub fn len(&self) -> u64 {
+        self.length
     }
 
     pub(crate) fn num_pages(&self) -> usize {
@@ -184,34 +304,37 @@ impl BitSet {
     }
 
     /// Sets the members of this set to the union of self and other.
-    pub(crate) fn union(&mut self, other: &BitSet) {
+    pub fn union(&mut self, other: &U32Set) {
         self.process(BitPage::union, other);
     }
 
     /// Sets the members of this set to the intersection of self and other.
-    pub(crate) fn intersect(&mut self, other: &BitSet) {
+    pub fn intersect(&mut self, other: &U32Set) {
         self.process(BitPage::intersect, other);
     }
 
     /// Sets the members of this set to self - other.
-    pub(crate) fn subtract(&mut self, other: &BitSet) {
+    pub fn subtract(&mut self, other: &U32Set) {
         self.process(BitPage::subtract, other);
     }
 
     /// Sets the members of this set to other - self.
-    pub(crate) fn reversed_subtract(&mut self, other: &BitSet) {
+    pub fn reversed_subtract(&mut self, other: &U32Set) {
         self.process(|a, b| BitPage::subtract(b, a), other);
     }
 
-    pub(crate) fn iter(&self) -> impl DoubleEndedIterator<Item = u32> + '_ {
+    /// Iterator over the members of this set. In sorted order (ascending).
+    pub fn iter(&self) -> impl DoubleEndedIterator<Item = u32> + '_ {
         self.iter_non_empty_pages().flat_map(|(major, page)| {
             let base = Self::major_start(major);
             page.iter().map(move |v| base + v)
         })
     }
 
-    /// Iterator over the members of this set that come after `value`.
-    pub(crate) fn iter_after(&self, value: u32) -> impl Iterator<Item = u32> + '_ {
+    /// Iterator over the members of this set starting from value.
+    ///
+    /// So value is included in the iterator if it's in the set.
+    pub fn iter_from(&self, value: u32) -> impl Iterator<Item = u32> + '_ {
         let major_value = Self::get_major_value(value);
         let result = self
             .page_map
@@ -236,7 +359,7 @@ impl BitSet {
                 .into_iter()
                 .flat_map(move |(page, major)| {
                     let base = Self::major_start(major);
-                    page.iter_after(value).map(move |v| base + v)
+                    page.iter_from(value).map(move |v| base + v)
                 });
 
         let follow_on_page_map_index = if partial_first_page {
@@ -261,8 +384,9 @@ impl BitSet {
         init_it.chain(follow_on_it)
     }
 
-    pub(crate) fn iter_ranges(&self) -> impl Iterator<Item = RangeInclusive<u32>> + '_ {
-        BitSetRangeIter::new(self)
+    /// Iterate over the ranges of contiguous values in this set.
+    pub fn iter_ranges(&self) -> impl Iterator<Item = RangeInclusive<u32>> + '_ {
+        U32SetRangeIter::new(self)
     }
 
     fn iter_pages(&self) -> impl DoubleEndedIterator<Item = (u32, &BitPage)> + '_ {
@@ -296,13 +420,11 @@ impl BitSet {
         (passthrough_left, passthrough_right)
     }
 
-    fn process<Op>(&mut self, op: Op, other: &BitSet)
+    fn process<Op>(&mut self, op: Op, other: &U32Set)
     where
         Op: Fn(&BitPage, &BitPage) -> BitPage,
     {
-        let (passthrough_left, passthrough_right) = BitSet::passthrough_behavior(&op);
-
-        self.mark_dirty();
+        let (passthrough_left, passthrough_right) = U32Set::passthrough_behavior(&op);
 
         let mut len_a = self.pages.len();
         let len_b = other.pages.len();
@@ -432,6 +554,7 @@ impl BitSet {
         }
 
         self.resize(new_count);
+        self.recompute_length();
     }
 
     fn compact(&mut self, new_len: usize) {
@@ -476,14 +599,6 @@ impl BitSet {
         self.pages.resize(new_len, BitPage::new_zeroes());
     }
 
-    fn mark_dirty(&mut self) {
-        self.len.set(u64::MAX);
-    }
-
-    fn is_dirty(&self) -> bool {
-        self.len.get() == u64::MAX
-    }
-
     /// Return the major value (top 23 bits) of the page associated with value.
     const fn get_major_value(value: u32) -> u32 {
         value >> PAGE_BITS_LOG_2
@@ -500,14 +615,19 @@ impl BitSet {
 
     /// Returns the index in `self.pages` (if it exists) for the page with the same major as `major_value`.
     fn page_index_for_major(&self, major_value: u32) -> Option<usize> {
+        self.page_map_index_for_major(major_value)
+            .map(|info_idx| self.page_map[info_idx].index as usize)
+    }
+
+    fn page_map_index_for_major(&self, major_value: u32) -> Option<usize> {
         self.page_map
             .binary_search_by(|probe| probe.major_value.cmp(&major_value))
             .ok()
-            .map(|info_idx| self.page_map[info_idx].index as usize)
     }
 
     /// Returns the index in `self.pages` for the page with the same major as `major_value`. Will create
     /// the page if it does not yet exist.
+    #[inline]
     fn ensure_page_index_for_major(&mut self, major_value: u32) -> usize {
         match self
             .page_map
@@ -527,13 +647,6 @@ impl BitSet {
         }
     }
 
-    /// Return a reference to the page that `value` resides in.
-    fn page_for(&self, value: u32) -> Option<&BitPage> {
-        let major_value = Self::get_major_value(value);
-        let pages_index = self.page_index_for_major(major_value)?;
-        self.pages.get(pages_index)
-    }
-
     /// Return a mutable reference to the page that `value` resides in.
     ///
     /// Insert a new page if it doesn't exist.
@@ -576,9 +689,9 @@ impl BitSet {
     }
 }
 
-impl Extend<u32> for BitSet {
+impl Extend<u32> for U32Set {
     fn extend<U: IntoIterator<Item = u32>>(&mut self, iter: U) {
-        let mut builder = BitSetBuilder::start(self);
+        let mut builder = U32SetBuilder::start(self);
         for val in iter {
             builder.insert(val);
         }
@@ -586,18 +699,18 @@ impl Extend<u32> for BitSet {
     }
 }
 
-/// This helper is used to construct [`BitSet`]'s from a stream of possibly sorted values.
+/// This helper is used to construct [`U32Set`]'s from a stream of possibly sorted values.
 /// It remembers the last page index to reduce the amount of page lookups needed when inserting
 /// sorted data. If given unsorted values it will still work correctly, but may be slower then just
 /// repeatedly calling `insert()` on the bitset.
-pub(crate) struct BitSetBuilder<'a> {
-    pub(crate) set: &'a mut BitSet,
+pub(crate) struct U32SetBuilder<'a> {
+    pub(crate) set: &'a mut U32Set,
     last_page_index: usize,
     last_major_value: u32,
 }
 
-impl<'a> BitSetBuilder<'a> {
-    pub(crate) fn start(set: &'a mut BitSet) -> Self {
+impl<'a> U32SetBuilder<'a> {
+    pub(crate) fn start(set: &'a mut U32Set) -> Self {
         Self {
             set,
             last_page_index: usize::MAX,
@@ -609,22 +722,24 @@ impl<'a> BitSetBuilder<'a> {
         // TODO(garretrieger): additional optimization ideas:
         // - Assuming data is sorted accumulate a single element mask and only commit it to the element
         //   once the next value passes the end of the element.
-        let major_value = BitSet::get_major_value(val);
+        let major_value = U32Set::get_major_value(val);
         if major_value != self.last_major_value {
             self.last_page_index = self.set.ensure_page_index_for_major(major_value);
             self.last_major_value = major_value;
         };
         if let Some(page) = self.set.pages.get_mut(self.last_page_index) {
-            page.insert_no_return(val);
+            self.set.length += page.insert(val) as u64;
         }
     }
 
-    pub(crate) fn finish(&mut self) {
-        self.set.mark_dirty();
+    pub(crate) fn finish(self) {
+        // we used to do some finalization and bookkeeping here, and we will
+        // want to again if we optimize the impl more.
     }
 }
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 struct PageInfo {
     // index into pages vector of this page
     index: u32,
@@ -632,13 +747,13 @@ struct PageInfo {
     major_value: u32,
 }
 
-impl Hash for BitSet {
+impl Hash for U32Set {
     fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
         self.iter_non_empty_pages().for_each(|t| t.hash(state));
     }
 }
 
-impl std::cmp::PartialEq for BitSet {
+impl std::cmp::PartialEq for U32Set {
     fn eq(&self, other: &Self) -> bool {
         let mut this = self.iter_non_empty_pages();
         let mut other = other.iter_non_empty_pages();
@@ -655,15 +770,15 @@ impl std::cmp::PartialEq for BitSet {
     }
 }
 
-impl std::cmp::Eq for BitSet {}
+impl std::cmp::Eq for U32Set {}
 
-impl std::cmp::PartialOrd for BitSet {
+impl std::cmp::PartialOrd for U32Set {
     fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
         Some(self.cmp(other))
     }
 }
 
-impl std::cmp::Ord for BitSet {
+impl std::cmp::Ord for U32Set {
     fn cmp(&self, other: &Self) -> core::cmp::Ordering {
         let this_it = self.iter();
         let other_it = other.iter();
@@ -680,18 +795,18 @@ impl std::cmp::Ord for BitSet {
     }
 }
 
-struct BitSetRangeIter<'a> {
-    set: &'a BitSet,
+struct U32SetRangeIter<'a> {
+    set: &'a U32Set,
     page_info_index: usize,
     page_iter: Option<RangeIter<'a>>,
 }
 
-impl<'a> BitSetRangeIter<'a> {
-    fn new(set: &'a BitSet) -> BitSetRangeIter<'a> {
-        BitSetRangeIter {
+impl<'a> U32SetRangeIter<'a> {
+    fn new(set: &'a U32Set) -> U32SetRangeIter<'a> {
+        U32SetRangeIter {
             set,
             page_info_index: 0,
-            page_iter: BitSetRangeIter::<'a>::page_iter(set, 0),
+            page_iter: U32SetRangeIter::<'a>::page_iter(set, 0),
         }
     }
 
@@ -702,10 +817,10 @@ impl<'a> BitSetRangeIter<'a> {
     }
 
     fn reset_page_iter(&mut self) {
-        self.page_iter = BitSetRangeIter::<'a>::page_iter(self.set, self.page_info_index);
+        self.page_iter = U32SetRangeIter::<'a>::page_iter(self.set, self.page_info_index);
     }
 
-    fn page_iter(set: &'a BitSet, page_info_index: usize) -> Option<RangeIter<'a>> {
+    fn page_iter(set: &'a U32Set, page_info_index: usize) -> Option<RangeIter<'a>> {
         set.page_map
             .get(page_info_index)
             .map(|pi| pi.index as usize)
@@ -716,7 +831,7 @@ impl<'a> BitSetRangeIter<'a> {
     fn next_range(&mut self) -> Option<RangeInclusive<u32>> {
         // TODO(garretrieger): don't recompute page start on each call.
         let page = self.set.page_map.get(self.page_info_index)?;
-        let page_start = BitSet::major_start(page.major_value);
+        let page_start = U32Set::major_start(page.major_value);
         self.page_iter
             .as_mut()?
             .next()
@@ -724,7 +839,7 @@ impl<'a> BitSetRangeIter<'a> {
     }
 }
 
-impl Iterator for BitSetRangeIter<'_> {
+impl Iterator for U32SetRangeIter<'_> {
     type Item = RangeInclusive<u32>;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -732,7 +847,7 @@ impl Iterator for BitSetRangeIter<'_> {
         let mut current_range = self.next_range();
         loop {
             let page = self.set.page_map.get(self.page_info_index)?;
-            let page_end = BitSet::major_end(page.major_value);
+            let page_end = U32Set::major_end(page.major_value);
 
             let Some(range) = current_range.clone() else {
                 // The current page has no more ranges, but there may be more pages.
@@ -774,24 +889,25 @@ mod test {
     use super::*;
     use std::collections::HashSet;
 
-    impl FromIterator<u32> for BitSet {
-        fn from_iter<I: IntoIterator<Item = u32>>(iter: I) -> Self {
-            let mut out = BitSet::empty();
-            out.extend(iter);
-            out
-        }
-    }
-
     #[test]
     fn len() {
-        let bitset = BitSet::empty();
+        let bitset = U32Set::empty();
         assert_eq!(bitset.len(), 0);
         assert!(bitset.is_empty());
     }
 
+    #[test]
+    fn from_iter() {
+        let mut expected = U32Set::empty();
+        expected.extend([2, 8, 13]);
+
+        assert_eq!(U32Set::from_iter([2, 8, 13]), expected);
+        assert_eq!(U32Set::from_iter([8, 2, 13]), expected);
+    }
+
     #[test]
     fn iter() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.insert(3);
         bitset.insert(8);
         bitset.insert(534);
@@ -805,7 +921,7 @@ mod test {
     }
 
     fn check_iter_ranges(ranges: Vec<RangeInclusive<u32>>) {
-        let mut set = BitSet::empty();
+        let mut set = U32Set::empty();
         for range in ranges.iter() {
             set.insert_range(*range.start()..=*range.end());
         }
@@ -833,7 +949,7 @@ mod test {
 
     #[test]
     fn iter_ranges_zero_pages() {
-        let mut set = BitSet::empty();
+        let mut set = U32Set::empty();
 
         set.insert(1000);
         set.insert_range(300..=511);
@@ -845,7 +961,7 @@ mod test {
 
     #[test]
     fn iter_backwards() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
 
         bitset.insert_range(1..=6);
         {
@@ -880,58 +996,77 @@ mod test {
     }
 
     #[test]
-    fn iter_after() {
-        let mut bitset = BitSet::empty();
+    fn iter_from() {
+        let mut bitset = U32Set::empty();
         bitset.extend([5, 7, 10, 1250, 1300, 3001]);
 
         assert_eq!(
-            bitset.iter_after(0).collect::<Vec<u32>>(),
+            bitset.iter_from(0).collect::<Vec<u32>>(),
             vec![5, 7, 10, 1250, 1300, 3001]
         );
 
         assert_eq!(
-            bitset.iter_after(5).collect::<Vec<u32>>(),
-            vec![7, 10, 1250, 1300, 3001]
+            bitset.iter_from(4).collect::<Vec<u32>>(),
+            vec![5, 7, 10, 1250, 1300, 3001]
         );
         assert_eq!(
-            bitset.iter_after(6).collect::<Vec<u32>>(),
+            bitset.iter_from(5).collect::<Vec<u32>>(),
+            vec![5, 7, 10, 1250, 1300, 3001]
+        );
+        assert_eq!(
+            bitset.iter_from(6).collect::<Vec<u32>>(),
             vec![7, 10, 1250, 1300, 3001]
         );
 
         assert_eq!(
-            bitset.iter_after(10).collect::<Vec<u32>>(),
+            bitset.iter_from(10).collect::<Vec<u32>>(),
+            vec![10, 1250, 1300, 3001]
+        );
+
+        assert_eq!(
+            bitset.iter_from(700).collect::<Vec<u32>>(),
             vec![1250, 1300, 3001]
         );
 
         assert_eq!(
-            bitset.iter_after(700).collect::<Vec<u32>>(),
+            bitset.iter_from(1250).collect::<Vec<u32>>(),
             vec![1250, 1300, 3001]
         );
-
         assert_eq!(
-            bitset.iter_after(1250).collect::<Vec<u32>>(),
+            bitset.iter_from(1251).collect::<Vec<u32>>(),
             vec![1300, 3001]
         );
 
-        assert_eq!(bitset.iter_after(3000).collect::<Vec<u32>>(), vec![3001]);
-        assert_eq!(bitset.iter_after(3001).collect::<Vec<u32>>(), vec![]);
-        assert_eq!(bitset.iter_after(3002).collect::<Vec<u32>>(), vec![]);
-        assert_eq!(bitset.iter_after(5000).collect::<Vec<u32>>(), vec![]);
-        assert_eq!(bitset.iter_after(u32::MAX).collect::<Vec<u32>>(), vec![]);
+        assert_eq!(bitset.iter_from(3000).collect::<Vec<u32>>(), vec![3001]);
+        assert_eq!(bitset.iter_from(3001).collect::<Vec<u32>>(), vec![3001]);
+        assert_eq!(bitset.iter_from(3002).count(), 0);
+        assert_eq!(bitset.iter_from(5000).count(), 0);
+        assert_eq!(bitset.iter_from(u32::MAX).count(), 0);
 
         bitset.insert(u32::MAX);
-        assert_eq!(bitset.iter_after(u32::MAX).collect::<Vec<u32>>(), vec![]);
         assert_eq!(
-            bitset.iter_after(u32::MAX - 1).collect::<Vec<u32>>(),
+            bitset.iter_from(u32::MAX).collect::<Vec<u32>>(),
+            vec![u32::MAX]
+        );
+        assert_eq!(
+            bitset.iter_from(u32::MAX - 1).collect::<Vec<u32>>(),
             vec![u32::MAX]
         );
 
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.extend([510, 511, 512]);
 
-        assert_eq!(bitset.iter_after(510).collect::<Vec<u32>>(), vec![511, 512]);
-        assert_eq!(bitset.iter_after(511).collect::<Vec<u32>>(), vec![512]);
-        assert_eq!(bitset.iter_after(512).collect::<Vec<u32>>(), vec![]);
+        assert_eq!(
+            bitset.iter_from(509).collect::<Vec<u32>>(),
+            vec![510, 511, 512]
+        );
+        assert_eq!(
+            bitset.iter_from(510).collect::<Vec<u32>>(),
+            vec![510, 511, 512]
+        );
+        assert_eq!(bitset.iter_from(511).collect::<Vec<u32>>(), vec![511, 512]);
+        assert_eq!(bitset.iter_from(512).collect::<Vec<u32>>(), vec![512]);
+        assert!(bitset.iter_from(513).collect::<Vec<u32>>().is_empty());
     }
 
     #[test]
@@ -939,10 +1074,10 @@ mod test {
         let values = [3, 8, 534, 700, 10000, 10001, 10002];
         let values_unsorted = [10000, 3, 534, 700, 8, 10001, 10002];
 
-        let mut s1 = BitSet::empty();
-        let mut s2 = BitSet::empty();
-        let mut s3 = BitSet::empty();
-        let mut s4 = BitSet::empty();
+        let mut s1 = U32Set::empty();
+        let mut s2 = U32Set::empty();
+        let mut s3 = U32Set::empty();
+        let mut s4 = U32Set::empty();
         assert_eq!(s1.len(), 0);
 
         s1.extend(values.iter().copied());
@@ -963,7 +1098,7 @@ mod test {
 
     #[test]
     fn insert_unordered() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
 
         assert!(!bitset.contains(0));
         assert!(!bitset.contains(768));
@@ -986,7 +1121,7 @@ mod test {
 
     #[test]
     fn remove() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
 
         assert!(bitset.insert(0));
         assert!(bitset.insert(511));
@@ -1009,7 +1144,7 @@ mod test {
 
     #[test]
     fn remove_all() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.extend([5, 7, 11, 18, 620, 2000]);
 
         assert_eq!(bitset.len(), 6);
@@ -1021,7 +1156,7 @@ mod test {
 
     #[test]
     fn remove_range() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.extend([5, 7, 11, 18, 511, 620, 1023, 1024, 1200]);
         assert_eq!(bitset.len(), 9);
         bitset.remove_range(7..=620);
@@ -1031,13 +1166,13 @@ mod test {
             vec![5, 1023, 1024, 1200]
         );
 
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.extend([5, 7, 11, 18, 511, 620, 1023, 1024, 1200]);
         bitset.remove_range(7..=1024);
         assert_eq!(bitset.len(), 2);
         assert_eq!(bitset.iter().collect::<Vec<u32>>(), vec![5, 1200]);
 
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.extend([5, 7, 11, 18, 511, 620, 1023, 1024, 1200]);
         bitset.remove_range(2000..=2100);
         assert_eq!(bitset.len(), 9);
@@ -1047,7 +1182,7 @@ mod test {
         );
 
         // Remove all from one page
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         bitset.extend([1001, 1002, 1003, 1004]);
         bitset.remove_range(1002..=1003);
         assert!(bitset.contains(1001));
@@ -1070,7 +1205,7 @@ mod test {
 
     #[test]
     fn remove_range_boundary() {
-        let mut set = BitSet::empty();
+        let mut set = U32Set::empty();
 
         set.remove_range(u32::MAX - 10..=u32::MAX);
         assert!(!set.contains(u32::MAX));
@@ -1089,7 +1224,7 @@ mod test {
 
     #[test]
     fn remove_to_empty_page() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
 
         bitset.insert(793);
         bitset.insert(43);
@@ -1102,7 +1237,7 @@ mod test {
 
     #[test]
     fn insert_max_value() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
         assert!(!bitset.contains(u32::MAX));
         assert!(bitset.insert(u32::MAX));
         assert!(bitset.contains(u32::MAX));
@@ -1110,16 +1245,54 @@ mod test {
         assert_eq!(bitset.len(), 1);
     }
 
+    #[test]
+    fn contains_index_cache() {
+        let mut bitset = U32Set::from_iter([10, 11, 12, 2023]);
+        // contains() internally uses a cache of last page index
+        // ensure that outward contains() returns are unnaffected
+        // by the ordering of calls.
+        assert!(!bitset.contains(9));
+        assert!(bitset.contains(10));
+        assert!(bitset.contains(11));
+        assert!(bitset.contains(12));
+
+        assert!(!bitset.contains(1200));
+        assert!(!bitset.contains(2022));
+        assert!(bitset.contains(2023));
+        assert!(!bitset.contains(2024));
+
+        assert!(bitset.contains(2023));
+        assert!(bitset.contains(11));
+
+        assert!(!bitset.contains(5000));
+        assert!(bitset.contains(11));
+        assert!(bitset.contains(2023));
+        assert!(bitset.contains(12));
+        assert!(!bitset.contains(2024));
+        assert!(!bitset.contains(13));
+
+        // Caching should also work correctly if the page map is modified between lookups
+        bitset.clear();
+        bitset.insert(2024);
+        bitset.insert(13);
+
+        assert!(bitset.contains(13));
+        assert!(!bitset.contains(12));
+
+        assert!(bitset.contains(2024));
+        assert!(!bitset.contains(2023));
+    }
+
     fn check_process<A, B, C, Op>(a: A, b: B, expected: C, op: Op)
     where
         A: IntoIterator<Item = u32>,
         B: IntoIterator<Item = u32>,
         C: IntoIterator<Item = u32>,
-        Op: Fn(&mut BitSet, &BitSet),
+        Op: Fn(&mut U32Set, &U32Set),
     {
-        let mut result = BitSet::from_iter(a);
-        let b_set = BitSet::from_iter(b);
-        let expected_set = BitSet::from_iter(expected);
+        let mut result = U32Set::from_iter(a);
+        let b_set = U32Set::from_iter(b);
+        let expected_set = U32Set::from_iter(expected);
         result.len();
 
         op(&mut result, &b_set);
@@ -1162,8 +1335,8 @@ mod test {
         check_process([5], [5, 1000], [1000], |a, b| a.reversed_subtract(b));
     }
 
-    fn set_for_range(first: u32, last: u32) -> BitSet {
-        let mut set = BitSet::empty();
+    fn set_for_range(first: u32, last: u32) -> U32Set {
+        let mut set = U32Set::empty();
         for i in first..=last {
             set.insert(i);
         }
@@ -1181,7 +1354,7 @@ mod test {
             (364, 700),
             (364, 6000),
         ] {
-            let mut set = BitSet::empty();
+            let mut set = U32Set::empty();
             set.len();
             set.insert_range(range.0..=range.1);
             assert_eq!(set, set_for_range(range.0, range.1), "{range:?}");
@@ -1191,7 +1364,7 @@ mod test {
 
     #[test]
     fn insert_range_on_existing() {
-        let mut set = BitSet::empty();
+        let mut set = U32Set::empty();
         set.insert(700);
         set.insert(2000);
         set.insert_range(32..=4000);
@@ -1201,7 +1374,7 @@ mod test {
 
     #[test]
     fn insert_range_max() {
-        let mut set = BitSet::empty();
+        let mut set = U32Set::empty();
         set.insert_range(u32::MAX..=u32::MAX);
         assert!(set.contains(u32::MAX));
         assert_eq!(set.len(), 1);
@@ -1209,7 +1382,7 @@ mod test {
 
     #[test]
     fn clear() {
-        let mut bitset = BitSet::empty();
+        let mut bitset = U32Set::empty();
 
         bitset.insert(13);
         bitset.insert(670);
@@ -1223,12 +1396,11 @@ mod test {
     }
 
     #[test]
-    #[allow(clippy::mutable_key_type)]
     fn hash_and_eq() {
-        let mut bitset1 = BitSet::empty();
-        let mut bitset2 = BitSet::empty();
-        let mut bitset3 = BitSet::empty();
-        let mut bitset4 = BitSet::empty();
+        let mut bitset1 = U32Set::empty();
+        let mut bitset2 = U32Set::empty();
+        let mut bitset3 = U32Set::empty();
+        let mut bitset4 = U32Set::empty();
 
         bitset1.insert(43);
         bitset1.insert(793);
@@ -1243,7 +1415,7 @@ mod test {
 
         bitset4.insert(0);
 
-        assert_eq!(BitSet::empty(), BitSet::empty());
+        assert_eq!(U32Set::empty(), U32Set::empty());
         assert_eq!(bitset1, bitset2);
         assert_ne!(bitset1, bitset3);
         assert_ne!(bitset2, bitset3);
@@ -1255,11 +1427,10 @@ mod test {
     }
 
     #[test]
-    #[allow(clippy::mutable_key_type)]
     fn hash_and_eq_with_empty_pages() {
-        let mut bitset1 = BitSet::empty();
-        let mut bitset2 = BitSet::empty();
-        let mut bitset3 = BitSet::empty();
+        let mut bitset1 = U32Set::empty();
+        let mut bitset2 = U32Set::empty();
+        let mut bitset3 = U32Set::empty();
 
         bitset1.insert(43);
 
@@ -1277,12 +1448,28 @@ mod test {
         assert!(set.contains(&bitset2));
     }
 
+    #[test]
+    fn hash_and_eq_ignore_cache() {
+        let bitset1 = U32Set::from_iter([5, 1027]);
+        let bitset2 = U32Set::from_iter([5, 1027]);
+
+        // Modify the internal last page index cache to point at different pages.
+        assert!(bitset1.contains(1027));
+        assert!(bitset2.contains(5));
+
+        // Hash, eq, cmp should be unnaffected:
+        assert_eq!(bitset1, bitset2);
+        assert!(matches!(bitset1.cmp(&bitset2), Ordering::Equal));
+        let set = HashSet::from([bitset1]);
+        assert!(set.contains(&bitset2));
+    }
+
     #[test]
     fn ordering() {
         macro_rules! assert_ord {
             ($lhs:expr, $rhs:expr, $ord:path) => {
                 assert_eq!(
-                    BitSet::from_iter($lhs).cmp(&BitSet::from_iter($rhs)),
+                    U32Set::from_iter($lhs).cmp(&U32Set::from_iter($rhs)),
                     $ord,
                     "{:?}, {:?}",
                     $lhs,
@@ -1310,4 +1497,37 @@ mod test {
             Ordering::Greater
         ); // out of order
     }
+
+    #[test]
+    fn intersects() {
+        macro_rules! assert_intersects {
+            ($lhs:path, $rhs:path, $expected:expr) => {
+                assert_eq!($lhs.intersects_set(&$rhs), $expected);
+                assert_eq!($rhs.intersects_set(&$lhs), $expected);
+            };
+        }
+
+        let a = U32Set::from_iter([2, 4, 5, 2057, 7000]);
+        let b = U32Set::from_iter([3]);
+        let c = U32Set::from_iter([2058]);
+        let d = U32Set::from_iter([2057, 3000]);
+        let e = U32Set::from_iter([3, 7000]);
+
+        assert_intersects!(a, b, false);
+        assert_intersects!(a, c, false);
+        assert_intersects!(e, d, false);
+
+        assert_intersects!(a, d, true);
+        assert_intersects!(a, e, true);
+        assert_intersects!(b, e, true);
+
+        // Check that page map population orderdoes not impact the check
+        let mut a = U32Set::empty();
+        a.insert(4000);
+        a.insert(0);
+
+        let b = U32Set::from_iter([4000]);
+
+        assert_intersects!(a, b, true);
+    }
 }
