@@ -5,7 +5,8 @@ include!("../../generated/generated_head.rs");
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_helpers::BeBuffer;
+    use crate::{FontRef, TableProvider};
+    use font_test_data::bebuffer::BeBuffer;
 
     #[test]
     fn smoke_text() {
@@ -19,11 +20,26 @@ mod tests {
             .extend([0u16, 12]) // mac_style / ppem
             .extend([2i16, 1, 0]);
 
-        let head = super::Head::read(buf.font_data()).unwrap();
+        let head = super::Head::read(buf.data().into()).unwrap();
         assert_eq!(head.version(), MajorMinor::VERSION_1_0);
         assert_eq!(head.font_revision(), Fixed::from_f64(2.8));
         assert_eq!(head.units_per_em(), 4096);
         assert_eq!(head.created().as_secs(), -500);
         assert_eq!(head.y_min(), -50);
+        assert_eq!(head.flags(), Flags::INSTRUCTIONS_MAY_ALTER_ADVANCE_WIDTH);
+    }
+
+    #[test]
+    fn flags() {
+        let head = FontRef::new(font_test_data::TINOS_SUBSET)
+            .unwrap()
+            .head()
+            .unwrap();
+        assert_eq!(
+            head.flags(),
+            Flags::BASELINE_AT_Y_0
+                | Flags::FORCE_INTEGER_PPEM
+                | Flags::INSTRUCTIONS_MAY_ALTER_ADVANCE_WIDTH
+        );
     }
 }
