@@ -1,7 +1,7 @@
 fn main() {
 	match serial2::SerialPort::available_ports() {
 		Err(e) => {
-			eprintln!("Failed to enumerate serial ports: {}", e);
+			eprintln!("Failed to enumerate serial ports: {e}");
 			std::process::exit(1);
 		},
 		Ok(ports) => {
