@@ -42,7 +42,7 @@ mod traversal;
 #[cfg(test)]
 mod traversal_tests;
 
-use raw::tables::colr;
+use raw::{tables::colr, FontRef};
 #[cfg(test)]
 use serde::{Deserialize, Serialize};
 
@@ -55,7 +55,9 @@ use read_fonts::{
 
 use std::{fmt::Debug, ops::Range};
 
-use traversal::{get_clipbox_font_units, traverse_v0_range, traverse_with_callbacks, VisitedSet};
+use traversal::{
+    get_clipbox_font_units, traverse_v0_range, traverse_with_callbacks, PaintDecycler,
+};
 
 pub use transform::Transform;
 
@@ -100,7 +102,7 @@ impl From<ReadError> for PaintError {
 ///
 /// All gradient callbacks of [`ColorPainter`] normalize color stops to be in the range of 0
 /// to 1.
-#[derive(Clone, PartialEq, Debug, Default)]
+#[derive(Copy, Clone, PartialEq, Debug, Default)]
 #[cfg_attr(test, derive(Serialize, Deserialize))]
 // This repr(C) is required so that C-side FFI's
 // are able to cast the ColorStop slice to a C-side array pointer.
@@ -261,7 +263,22 @@ pub trait ColorPainter {
     /// Open a new layer, and merge the layer down using `composite_mode` when
     /// [`pop_layer`](ColorPainter::pop_layer) is called, signalling that this layer is done drawing.
     fn push_layer(&mut self, composite_mode: CompositeMode);
-    fn pop_layer(&mut self);
+
+    /// Merge the pushed layer down using `composite_mode` passed to the matching
+    /// [`push_layer`](ColorPainter::push_layer).
+    fn pop_layer(&mut self) {}
+
+    /// Alternative version of [`push_layer`](ColorPainter::push_layer) where the
+    /// `composite_mode` is also passed to the method. This is useful for
+    /// graphics libraries that need the compositing mode at layer pop time
+    /// and do not want to manually track the mode.
+    ///
+    /// Only one of [`pop_layer`](ColorPainter::pop_layer) or this method
+    /// need to be implemented. By default, this simply calls
+    /// [`pop_layer`](ColorPainter::pop_layer).
+    fn pop_layer_with_mode(&mut self, _composite_mode: CompositeMode) {
+        self.pop_layer();
+    }
 }
 
 /// Distinguishes available color glyph formats.
@@ -347,7 +364,9 @@ impl<'a> ColorGlyph<'a> {
         location: impl Into<LocationRef<'a>>,
         painter: &mut impl ColorPainter,
     ) -> Result<(), PaintError> {
-        let instance = instance::ColrInstance::new(self.colr.clone(), location.into().coords());
+        let instance =
+            instance::ColrInstance::new(self.colr.clone(), location.into().effective_coords());
+        let mut resolved_stops = traversal::ColorStopVec::default();
         match &self.root_paint_ref {
             ColorGlyphRoot::V1Paint(paint, paint_id, glyph_id, _) => {
                 let clipbox = get_clipbox_font_units(&instance, *glyph_id);
@@ -356,13 +375,14 @@ impl<'a> ColorGlyph<'a> {
                     painter.push_clip_box(rect);
                 }
 
-                let mut visited_set = VisitedSet::default();
-                visited_set.insert(*paint_id);
+                let mut decycler = PaintDecycler::default();
+                let mut cycle_guard = decycler.enter(*paint_id)?;
                 traverse_with_callbacks(
                     &resolve_paint(&instance, paint)?,
                     &instance,
                     painter,
-                    &mut visited_set,
+                    &mut cycle_guard,
+                    &mut resolved_stops,
                     0,
                 )?;
 
@@ -388,7 +408,7 @@ pub struct ColorGlyphCollection<'a> {
 
 impl<'a> ColorGlyphCollection<'a> {
     /// Creates a new collection of paintable color glyphs for the given font.
-    pub fn new(font: &impl TableProvider<'a>) -> Self {
+    pub fn new(font: &FontRef<'a>) -> Self {
         let colr = font.colr().ok();
         let upem = font.head().map(|h| h.units_per_em());
 
