@@ -292,7 +292,7 @@ impl WindowsResource {
     ///
     /// It is possible to use arbitrary field names but Windows Explorer and other
     /// tools might not show them.
-    pub fn set<'a>(&mut self, name: &'a str, value: &'a str) -> &mut Self {
+    pub fn set(&mut self, name: &str, value: &str) -> &mut Self {
         self.properties.insert(name.to_string(), value.to_string());
         self
     }
@@ -312,7 +312,7 @@ impl WindowsResource {
     ///
     /// If it is left unset, it will look up a path in the registry,
     /// i.e. `HKLM\SOFTWARE\Microsoft\Windows Kits\Installed Roots`
-    pub fn set_toolkit_path<'a>(&mut self, path: &'a str) -> &mut Self {
+    pub fn set_toolkit_path(&mut self, path: &str) -> &mut Self {
         self.toolkit_path = PathBuf::from(path);
         self
     }
@@ -321,7 +321,7 @@ impl WindowsResource {
     ///
     /// # Example
     ///
-    /// ```
+    /// ```no_run
     /// extern crate winapi;
     /// extern crate winresource;
     /// # use std::io;
@@ -375,7 +375,7 @@ impl WindowsResource {
     /// or relative to the projects root.
     ///
     /// Equivalent to `set_icon_with_id(path, "1")`.
-    pub fn set_icon<'a>(&mut self, path: &'a str) -> &mut Self {
+    pub fn set_icon(&mut self, path: &str) -> &mut Self {
         const DEFAULT_APPLICATION_ICON_ID: &str = "1";
 
         self.set_icon_with_id(path, DEFAULT_APPLICATION_ICON_ID)
@@ -428,7 +428,7 @@ impl WindowsResource {
     ///    .set_icon_with_id("icon3.icon", "3")
     ///    // ...
     /// ```
-    pub fn set_icon_with_id<'a>(&mut self, path: &'a str, name_id: &'a str) -> &mut Self {
+    pub fn set_icon_with_id(&mut self, path: &str, name_id: &str) -> &mut Self {
         self.icons.push(Icon {
             path: path.into(),
             name_id: name_id.into(),
@@ -466,7 +466,7 @@ impl WindowsResource {
     /// "#);
     /// }
     /// ```
-    pub fn set_manifest<'a>(&mut self, manifest: &'a str) -> &mut Self {
+    pub fn set_manifest(&mut self, manifest: &str) -> &mut Self {
         self.manifest_file = None;
         self.manifest = Some(manifest.to_string());
         self
@@ -478,7 +478,7 @@ impl WindowsResource {
     ///
     /// [`set_manifest()`]: #method.set_manifest
     /// [`set_icon()`]: #method.set_icon
-    pub fn set_manifest_file<'a>(&mut self, file: &'a str) -> &mut Self {
+    pub fn set_manifest_file(&mut self, file: &str) -> &mut Self {
         self.manifest_file = Some(file.to_string());
         self.manifest = None;
         self
@@ -570,7 +570,7 @@ impl WindowsResource {
     /// We will neither modify this file nor parse its contents. This function
     /// simply replaces the internally generated resource file that is passed to
     /// the compiler. You can use this function to write a resource file yourself.
-    pub fn set_resource_file<'a>(&mut self, path: &'a str) -> &mut Self {
+    pub fn set_resource_file(&mut self, path: &str) -> &mut Self {
         self.rc_file = Some(path.to_string());
         self
     }
@@ -605,7 +605,7 @@ impl WindowsResource {
     /// # }
     /// # Ok::<_, std::io::Error>(())
     /// ```
-    pub fn append_rc_content<'a>(&mut self, content: &'a str) -> &mut Self {
+    pub fn append_rc_content(&mut self, content: &str) -> &mut Self {
         if !(self.append_rc_content.ends_with('\n') || self.append_rc_content.is_empty()) {
             self.append_rc_content.push('\n');
         }
@@ -617,17 +617,30 @@ impl WindowsResource {
     ///
     /// As a default, we use `%OUT_DIR%` set by cargo, but it may be necessary to override the
     /// the setting.
-    pub fn set_output_directory<'a>(&mut self, path: &'a str) -> &mut Self {
+    pub fn set_output_directory(&mut self, path: &str) -> &mut Self {
         self.output_directory = path.to_string();
         self
     }
 
-    fn compile_with_toolkit_gnu<'a>(&self, input: &'a str, output_dir: &'a str) -> io::Result<()> {
+    fn compile_with_toolkit_gnu(&self, input: &str, output_dir: &str) -> io::Result<()> {
+        let bfd_target = match env::var("CARGO_CFG_TARGET_ARCH").unwrap().as_str() {
+            "x86_64" => &["--target", "pe-x86-64"][..],
+            "x86" => &["--target", "pe-i386"],
+            // The case below would be correct for AArch64, but LLVM's windres does not handle
+            // the conversion from this BFD target to its LLVM target, treating it as a native
+            // LLVM target instead, which causes an error. Obviously, passing a LLVM target
+            // is not portable to the binutils windres implementation. So, to prevent breaking
+            // native AArch64 compilation with the LLVM toolchain, let's fall back to the default
+            // target, which would provide the highest success rate
+            //"aarch64" => &["--target", "pe-aarch64-little"],
+            _ => &[], // A quite strange arch - use the default windres target and hope for the best
+        };
         let output = PathBuf::from(output_dir).join("resource.o");
         let input = PathBuf::from(input);
         let status = process::Command::new(&self.windres_path)
             .current_dir(&self.toolkit_path)
             .arg(format!("-I{}", env::var("CARGO_MANIFEST_DIR").unwrap()))
+            .args(bfd_target)
             .arg(format!("{}", input.display()))
             .arg(format!("{}", output.display()))
             .status()?;
@@ -695,14 +708,7 @@ impl WindowsResource {
         }
     }
 
-    #[cfg(not(windows))]
-    #[inline(always)]
-    fn compile_with_toolkit_msvc<'a>(&self, input: &'a str, output_dir: &'a str) -> io::Result<()> {
-        self.compile_with_toolkit_gnu(input, output_dir)
-    }
-
-    #[cfg(windows)]
-    fn compile_with_toolkit_msvc<'a>(&self, input: &'a str, output_dir: &'a str) -> io::Result<()> {
+    fn compile_with_toolkit_msvc(&self, input: &str, output_dir: &str) -> io::Result<()> {
         let rc_exe = if let Some(rc_path) = std::env::var_os("RC_PATH") {
             PathBuf::from(rc_path)
         } else if cfg!(unix) {
@@ -732,10 +738,18 @@ impl WindowsResource {
             command.arg(format!("/I{}", root.join("um").display()));
             command.arg(format!("/I{}", root.join("shared").display()));
         }
-        let status = command
-            .arg(format!("/fo{}", output.display()))
-            .arg(format!("{}", input.display()))
-            .output()?;
+
+        command.arg(format!("/fo{}", output.display()));
+
+        if cfg!(unix) {
+            // Fix for https://github.com/llvm/llvm-project/issues/63426
+            command.args(["/C", "65001"]);
+
+            // Ensure paths starting with "/Users" on macOS are not interpreted as a /U option
+            command.arg("--");
+        }
+
+        let status = command.arg(format!("{}", input.display())).output()?;
 
         println!(
             "RC Output:\n{}\n------",
